@@ -37,26 +37,6 @@ namespace detail
 
 using cxxint_t = typename detail::IndexT<sizeof(long)>::type;
 
-namespace detail
-{
-  template<bool, typename T1, typename T2>
-  struct StaticIf;
-
-  // non-bits
-  template<typename T1, typename T2>
-  struct StaticIf<false, T1, T2>
-  {
-    typedef T2 type;
-  };
-
-  // bits type
-  template<typename T1, typename T2>
-  struct StaticIf<true, T1, T2>
-  {
-    typedef T1 type;
-  };
-}
-
 JLCXX_API void protect_from_gc(jl_value_t* v);
 JLCXX_API void unprotect_from_gc(jl_value_t* v);
 JLCXX_API void cxx_root_scanner(int);
@@ -155,7 +135,7 @@ namespace detail
 template<typename T> using supertype = typename detail::_get_supertype<T>::type;
 
 /// Remove reference and const from a type
-template<typename T> using remove_const_ref = typename std::remove_const<typename std::remove_reference<T>::type>::type;
+template<typename T> using remove_const_ref = std::remove_const_t<std::remove_reference_t<T>>;
 
 /// Indicate if a type is a smart pointer
 template<typename T> struct IsSmartPointerType
@@ -194,31 +174,9 @@ inline CppT* extract_pointer_nonull(const WrappedCppPtr& p)
   return extract_pointer<CppT>(p);
 }
 
-namespace detail
-{
-  template<typename T>
-  struct unused_type
-  {
-  };
-
-  template<typename T1, typename T2>
-  struct DefineIfDifferent
-  {
-    typedef T1 type;
-  };
-
-  template<typename T>
-  struct DefineIfDifferent<T,T>
-  {
-    typedef unused_type<T> type;
-  };
-
-  template<typename T1, typename T2> using define_if_different = typename DefineIfDifferent<T1,T2>::type;
-}
-
 // By default, fundamental and "POD" types are mapped directly
 template<typename T>
-struct IsMirroredType : std::bool_constant<(!std::is_class<T>::value || (std::is_standard_layout<T>::value && std::is_trivial<T>::value)) && !IsSmartPointerType<T>::value>
+struct IsMirroredType : std::bool_constant<(!std::is_class_v<T> || (std::is_standard_layout_v<T> && std::is_trivial_v<T>)) && !IsSmartPointerType<T>::value>
 {
 };
 
@@ -271,8 +229,14 @@ struct MappingTrait<void*>
   using type = DirectPtrTrait;
 };
 
+template<>
+struct MappingTrait<FILE*>
+{
+  using type = DirectPtrTrait;
+};
+
 template<typename T>
-struct MappingTrait<T, typename std::enable_if<!IsMirroredType<T>::value && !IsSmartPointerType<T>::value>::type>
+struct MappingTrait<T, std::enable_if_t<!IsMirroredType<T>::value && !IsSmartPointerType<T>::value>>
 {
   using type = CxxWrappedTrait<NoCxxWrappedSubtrait>;
 };
@@ -398,7 +362,7 @@ CachedDatatype& stored_type()
 template<typename T>
 void set_julia_type(jl_datatype_t* dt, bool protect = true)
 {
-  using nonconst_t = typename std::remove_const<T>::type;
+  using nonconst_t = std::remove_const_t<T>;
   CachedDatatype& cache = stored_type<nonconst_t>();
   if(cache.get_dt() != nullptr)
   {
@@ -424,7 +388,7 @@ public:
 template<typename T>
 inline jl_datatype_t* julia_type()
 {
-  using nonconst_t = typename std::remove_const<T>::type;
+  using nonconst_t = std::remove_const_t<T>;
   jl_datatype_t* dt = stored_type<nonconst_t>().get_dt();
   if(dt == nullptr)
   {
@@ -437,7 +401,7 @@ inline jl_datatype_t* julia_type()
 template <typename T>
 bool has_julia_type()
 {
-  using nonconst_t = typename std::remove_const<T>::type;
+  using nonconst_t = std::remove_const_t<T>;
   return stored_type<nonconst_t>().get_dt() != nullptr;
 }
 
@@ -445,7 +409,7 @@ bool has_julia_type()
 template <typename T>
 void create_julia_type()
 {
-  using nonconst_t = typename std::remove_const<T>::type;
+  using nonconst_t = std::remove_const_t<T>;
   jl_datatype_t* result = julia_type_factory<nonconst_t>::julia_type();
   if(!has_julia_type<nonconst_t>())
   {
@@ -456,7 +420,7 @@ void create_julia_type()
 template<typename T>
 void create_if_not_exists()
 {
-  using nonconst_t = typename std::remove_const<T>::type;
+  using nonconst_t = std::remove_const_t<T>;
   static bool exists = false;
   if(!exists)
   {
@@ -565,6 +529,16 @@ struct julia_type_factory<jl_value_t*>
   }
 };
 
+template<>
+struct julia_type_factory<FILE*>
+{
+  static inline jl_datatype_t* julia_type()
+  {
+    static jl_datatype_t* result = (jl_datatype_t*)jl_apply_type1((jl_value_t*)jl_pointer_type, jlcxx::julia_type("FILE", "Libc"));
+    return result;
+  }
+};
+
 /// Base class to specialize for conversion to C++
 template<typename CppT, typename TraitT=mapping_trait<CppT>>
 struct ConvertToCpp
@@ -659,7 +633,7 @@ BoxedValue<T> boxed_cpp_pointer(T* cpp_ptr, jl_datatype_t* dt, bool add_finalize
 template<typename T>
 BoxedValue<T> julia_owned(T* cpp_ptr)
 {
-  static_assert(!std::is_fundamental<T>::value, "Ownership can't be transferred for fundamental types");
+  static_assert(!std::is_fundamental_v<T>, "Ownership can't be transferred for fundamental types");
   const bool finalize = true;
   return boxed_cpp_pointer(cpp_ptr, julia_type<T>(), finalize);
 }
@@ -673,8 +647,8 @@ struct ConvertToJulia
   template<typename CppT>
   jl_value_t* operator()(CppT&& cpp_val) const
   {
-    static_assert(std::is_same<static_julia_type<T>, WrappedCppPtr>::value, "No appropriate specialization for ConvertToJulia");
-    static_assert(std::is_class<T>::value, "Need class type for conversion");
+    static_assert(std::is_same_v<static_julia_type<T>, WrappedCppPtr>, "No appropriate specialization for ConvertToJulia");
+    static_assert(std::is_class_v<T>, "Need class type for conversion");
     return julia_owned(new T(std::move(cpp_val)));
   }
 };
@@ -684,7 +658,7 @@ struct ConvertToJulia<T&, WrappedPtrTrait>
 {
   WrappedCppPtr operator()(T& cpp_val) const
   {
-    return {reinterpret_cast<void*>(const_cast<typename std::remove_const<T>::type*>(&cpp_val))};
+    return {reinterpret_cast<void*>(const_cast<std::remove_const_t<T>*>(&cpp_val))};
   }
 };
 
@@ -693,7 +667,7 @@ struct ConvertToJulia<T*, WrappedPtrTrait>
 {
   WrappedCppPtr operator()(T* cpp_val) const
   {
-    return {reinterpret_cast<void*>(const_cast<typename std::remove_const<T>::type*>(cpp_val))};
+    return {reinterpret_cast<void*>(const_cast<std::remove_const_t<T>*>(cpp_val))};
   }
 };
 
